﻿using Hotline.Share.Dtos.DataSharing.Province;
using IdentityModel.Client;
using Microsoft.Extensions.DependencyInjection;
using System.Net.Http.Headers;
using XF.Domain.Cache;
using XF.Domain.Exceptions;

namespace Hotline.DataSharing.Province
{
    /// <summary>
    /// 
    /// </summary>
    public abstract class ProvinceClient //: ISingletonDependency, ISelfDependency
    {
        private static readonly string KeyToken = "KeyToken";
        private readonly IServiceScopeFactory _scopeFactory;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="scopeFactory"></param>
        public ProvinceClient(IServiceScopeFactory scopeFactory)
        {
            _scopeFactory = scopeFactory;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="TRequest"></typeparam>
        /// <typeparam name="TResponse"></typeparam>
        /// <param name="request"></param>
        /// <param name="baseAddress"></param>
        /// <param name="configProvince"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task<TResponse?> RequestAsync<TRequest, TResponse>(
            TRequest request,
            string baseAddress,
            ConfigurationProvince configProvince,
            CancellationToken cancellationToken)
            where TRequest : IProvinceRequest, new()
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            //var channelconfigManager = provider.GetRequiredService<IChannelConfigurationManager>();
            var httpInvoker = provider.GetRequiredService<IHttpInvoker>();
            var cacheToken = provider.GetRequiredService<ITypedCache<TokenInfo>>();

            //不同的批次获取TOKEN的地址不一样
            string cacheKey = "";
            if (baseAddress == configProvince.HuiJu)
                cacheKey = KeyToken + "_HuiJu";
            else
                cacheKey = KeyToken + "_XieTong";

            var token = cacheToken.GetOrSet(cacheKey,
                d => GetTokenAsync(baseAddress, cancellationToken).GetAwaiter().GetResult(),
                TimeSpan.FromMinutes(28));
            //var configProvince = channelconfigManager.GetConfigurationProvince();
            request.BuildClientInfo(configProvince.ClientId, configProvince.ClientSecret);

            //todo token失效重新获取token再次请求，返回参数缺少该状态，暂无法处理

            return await httpInvoker.RequestAsync<TRequest, TResponse>(request,
                d => d.SetHttpClient(baseAddress, token?.AccessToken ?? string.Empty),
                cancellationToken);
        }

        /// <summary>
        /// 附件获取Token
        /// </summary>
        /// <param name="baseAddress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task<TokenInfo> GetToken(string baseAddress, CancellationToken cancellationToken)
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            var cacheToken = provider.GetRequiredService<ITypedCache<TokenInfo>>();
            string cacheKey = KeyToken + "_XieTong";

            var token = cacheToken.GetOrSet(cacheKey,
                d => GetTokenAsync(baseAddress, cancellationToken).GetAwaiter().GetResult(),
                TimeSpan.FromMinutes(28));
            return token;
        }

        /// <summary>
        /// 获取token
        /// </summary>
        /// <param name="baseAddress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        private async Task<TokenInfo> GetTokenAsync(string baseAddress, CancellationToken cancellationToken)
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            var channelconfigManager = provider.GetRequiredService<IChannelConfigurationManager>();
            var httpInvoker = provider.GetRequiredService<IHttpInvoker>();

            var configProvince = channelconfigManager.GetConfigurationProvince();
            //   var baseAddress = baseAddress;// configProvince.XieTong;// configProvince.HuiJu;
            if (!baseAddress.EndsWith('/'))
                baseAddress += "/";
            var request = new ClientCredentialsTokenRequest
            {
                ClientId = configProvince.ClientId,
                ClientSecret = configProvince.ClientSecret,
                Address = $"{baseAddress}oauth2/token",
                Headers =
                {
                    Accept =
                    {
                        MediaTypeWithQualityHeaderValue.Parse("application/x-www-form-urlencoded")
                    }
                }

            };
            var tokenResponse = await httpInvoker.GetTokenAsync(request, cancellationToken);
            if (tokenResponse is null)
                throw new UserFriendlyException("获取token请求失败");
            if (tokenResponse.IsError)
                throw new UserFriendlyException("获取token请求失败");

            var tokenInfo = System.Text.Json.JsonSerializer.Deserialize<TokenRsp>(tokenResponse.Raw);
            if (tokenInfo?.TokenInfo is null)
                throw new UserFriendlyException("token解析失败");
            return tokenInfo.TokenInfo;
        }
    }
}
